﻿using System.Windows.Forms;
using SDRSharp.Common;
using SDRSharp.Radio;

namespace SDRSharp.NoiseBlanker
{
    public class BasebandNoiseBlankerPlugin: ISharpPlugin
    {
        private const string _displayName = "Baseband Noise Blanker";
        private ISharpControl _control;
        private NoiseBlankerProcessor _processor;
                        
        private ProcessorPanel _guiControl;
        
        public string DisplayName
        {
            get { return _displayName; }
        }

        public UserControl Gui
        {
            get { return _guiControl; }
        }

        public void Initialize(ISharpControl control)
        {
            _control = control;

            _processor = new NoiseBlankerProcessor();
            _processor.Enabled = Utils.GetBooleanSetting("nb.bb.enabled");
            _processor.NoiseThreshold = Utils.GetDoubleSetting("nb.bb.threshold", 5);
            _processor.PulseWidth = Utils.GetDoubleSetting("nb.bb.pulseWidth", 50);
            _processor.LookupWindow = Utils.GetDoubleSetting("nb.bb.lookupWindow", 20);
            _guiControl = new ProcessorPanel(_processor);

            _control.RegisterStreamHook(_processor, ProcessorType.RawIQ);
        }

        public void Close()
        {
            Utils.SaveSetting("nb.bb.enabled", _processor.Enabled);
            Utils.SaveSetting("nb.bb.threshold", _processor.NoiseThreshold);
            Utils.SaveSetting("nb.bb.pulseWidth", _processor.PulseWidth);
            Utils.SaveSetting("nb.bb.lookupWindow", _processor.LookupWindow);
        }        
    }

    public class DemodulatorNoiseBlankerPlugin : ISharpPlugin
    {
        private const string _displayName = "Demodulator Noise Blanker";
        private ISharpControl _control;
        private NoiseBlankerProcessor _processor;

        private ProcessorPanel _guiControl;

        public string DisplayName
        {
            get { return _displayName; }
        }

        public UserControl Gui
        {
            get { return _guiControl; }
        }

        public void Initialize(ISharpControl control)
        {
            _control = control;

            _processor = new NoiseBlankerProcessor();
            _processor.Enabled = Utils.GetBooleanSetting("nb.demod.enabled");
            _processor.NoiseThreshold = Utils.GetDoubleSetting("nb.demod.threshold", 5);
            _processor.PulseWidth = Utils.GetDoubleSetting("nb.demod.pulseWidth", 50);
            _processor.LookupWindow = Utils.GetDoubleSetting("nb.demod.lookupWindow", 20);
            _guiControl = new ProcessorPanel(_processor);

            _control.RegisterStreamHook(_processor, ProcessorType.DemodulatorOutput);
        }

        public void Close()
        {
            Utils.SaveSetting("nb.demod.enabled", _processor.Enabled);
            Utils.SaveSetting("nb.demod.threshold", _processor.NoiseThreshold);
            Utils.SaveSetting("nb.demod.pulseWidth", _processor.PulseWidth);
            Utils.SaveSetting("nb.demod.lookupWindow", _processor.LookupWindow);
        }
    }
}
